/*************************************************************************
Title........ graphics.h
Date......... Wed Sep 29 08:47:34  1999
Author....... J.M.Tarrant
*************************************************************************/
/****	Defines */
/*
	Text colours
	NOTE: Only valid if standard VGA palette loaded.
*/
#define	 BLACK			0
#define	 BLUE			1
#define	 GREEN			2
#define	 CYAN			3
#define	 RED			4
#define	 MAGENTA		5
#define	 BROWN			6
#define	 LIGHTGRAY		7
#define	 DARKGRAY		8
#define	 LIGHTBLUE		9
#define	 LIGHTGREEN		10
#define	 LIGHTCYAN		11
#define	 LIGHTRED		12
#define	 LIGHTMAGENTA	13
#define	 YELLOW			14
#define	 WHITE			15

#define TRANSPARENT		255
#define	NORMAL			0

/****	Type Definitions */

typedef	struct	adb
	{
	LONG	screenaddr[2];
	BYTE	currentscreen;
	BYTE	screenrate;
	BYTE	screencount;
	} ADB;

enum {SINGLE_BUFFER,FRAME60HZ,FRAME30HZ,FRAME20HZ,FRAME15HZ,FRAME12HZ,FRAME10HZ,FRAME5HZ=12,FRAME3HZ=20,FRAME2HZ=30,FRAME1HZ=60};

typedef	struct	point
	{
	WORD	x;
	WORD	y;
	} XY;

typedef	struct	window
	{
	WORD	x1;
	WORD	x2;
	WORD	y1;
	WORD	y2;
	} WINDOW;


typedef	struct rgb
	{
	UBYTE	red;
	UBYTE	green;
	UBYTE	blue;
	} RGB;


typedef	struct graphicsservices
	{
	void  (*init_display_mode)(const WORD *reg_table);
	void  (*load_palette)(const RGB* pal_table);
	void  (*get_palette)(UBYTE entry,RGB* col);
    void  (*set_palette)(UBYTE entry,RGB* col);
	void  (*set_display_start)(ULONG screen_start_address);
	void  (*video_on)(void);
	void  (*video_blank)(void);
	void  (*load_font)(const ULONG *font);
	void  (*setcursor)(LONG screen,WORD x,WORD y);
	void  (*textcolor)(UWORD col);
	void  (*textbackground)(UWORD col);
	void  (*vputchar)(UBYTE ch);
	void  (*putpixel)(LONG screen,LONG x, LONG y, UWORD color);
	UWORD (*getpixel)(LONG screen,LONG x, LONG y);
	void  (*restore_background)(LONG screenbase,LONG x, LONG y, LONG save_address,LONG xsize,LONG ysize);
	void  (*save_background)(LONG screenbase,LONG x, LONG y, LONG save_address,LONG xsize,LONG ysize);
	void  (*put_sprite)(LONG screenbase,LONG x,LONG y,LONG image_start,LONG xsize,LONG ysize,LONG mode);
	void  (*putimage)(LONG screenbase, LONG x, LONG y,const WORD* image);
	void  (*putwinimage)(LONG screenbase,WINDOW* win,LONG wx,LONG wy,const UWORD* image);
	UBYTE (*blt_busy)();
	LONG  (*load_sprite)(const UBYTE* startaddr,LONG offset,LONG size);
	LONG  (*loadimage)(const UBYTE* startaddr,LONG offset,LONG size);
	LONG  (*loadimage_800x600)(const UBYTE* startaddr,LONG offset,LONG size);
	BYTE  (*screenfree)(BYTE scrn);
	void  (*setdoublebuffer)(LONG scrn0,LONG scrn1,BYTE rate);
	LONG  (*getscreen)(BYTE scrn);

/* Cremson Advanced Driver Functions */
	void  (*initdrawframe)(WORD mode, LONG base, WORD xres);
	void  (*initlayer)(void* layer, BYTE colour_mode, BYTE flip_mode, WORD transparent_colour, WORD width, WORD height, LONG orig_adr0, LONG disp_adr0, LONG orig_adr1, LONG disp_adr1);
	void  (*enablelayer)(UBYTE select);
	void  (*disablelayer)(UBYTE select);
	void  (*scrolllayer)(void* layer, ULONG xoff, ULONG yoff);
	void  (*clearlayer)(void* layer, BYTE scrn, LONG width);
	UBYTE (*vsync)();
	void  (*clear_background)(LONG screenbase, LONG xsize, LONG ysize);
	void  (*clear_sprite)(LONG screenbase, LONG x, LONG y, LONG xsize, LONG ysize);
	void  (*screentoggle)(WORD enable);
	void  (*screenpartitionx)(WORD x);
	void  (*vputstr)(BYTE *str);
	void  (*clear_area)(void* layer, BYTE scrn, LONG x, LONG y, LONG xsize, LONG ysize);
	void  (*windowlayer)(WORD xpos, WORD ypos, WORD width, WORD height);
	void  (*load_cpalette)(const RGB* pal_table);
	void  (*get_cpalette)(UBYTE entry,RGB* col);
    void  (*set_cpalette)(UBYTE entry,RGB* col);
	void  (*load_mbpalette)(const RGB* pal_table);
	void  (*get_mbpalette)(UBYTE entry,RGB* col);
    void  (*set_mbpalette)(UBYTE entry,RGB* col);
	void  (*blit)(LONG saddr,LONG sstride,WORD srx,WORD sry,LONG daddr,LONG dstride,WORD drx,WORD dry,WORD brsizex,WORD brsizey);
	void  (*byteorder)(UBYTE order);
	LONG  (*make_sprite)(WORD* sourceaddr,LONG offset);
	void  (*set_color_mode)(UBYTE mode);
	UBYTE (*get_color_mode)();
	} GRAPHICSSERVICES;

/****	Prototypes */
/*************************************************************************/
