/*************************************************************************
Title........ stepper.h
Date......... Wed Sep 15 15:56:45  1999
Author....... J.M.Tarrant
*************************************************************************/
/****	Defines */

	#define	RAMP_INI			0
	#define	RAMP_SYS			1
	#define RAMP_69				2
	#define RAMP_78				3
	#define RAMP_89				4
	#define RAMP_96				5
	#define RAMP_104			6
	#define	RAMP_NUDGE1			7
	#define	RAMP_NUDGE2			8
	#define	RAMP_NUDGE3			9
	#define RAMP_WOBBLE			10
	#define RAMP_HSWOBBLE		11
	#define RAMP_SKILLSLOW		12
	#define RAMP_SKILLMED		13
	#define RAMP_SKILLFAST		14
	#define RAMP_USER			15
	#define MAX_RAMPS			16

	#define	TAB1_MASK			0xFF00
	#define	CORRECT_MASK		0x00FF
	#define	full_step_mask		0x03		/* reels .h */
	#define	half_step_mask		0x03		/* reels .h */
	#define	EOF_STEPPERS		0x00

	#define	SKS_STOP			0x01
	#define	SKS_CORRECTED		0x02

	#define	ERR_BADOPTO			0x01
	#define	ERR_BADREEL			0x02

	#define OPTO_COUNT_MIN		30	/*	when a reel spins very fast and has many steps in one rotation, the driver
									 detects n times (depending on the reel) the same opto position. This define a nb
									 of steps to count before incrementing an opto position. This count will
									 stop the driver from counting the same opto n times each rotation as only one opto
									 should be seen	 each rotation*/

/****	Type Definitions */

typedef	struct ramp
		{
		const WORD	*pattern_tbl;						//set of value to send to the coil controling the reel movement
		WORD		pixmask;							//mask
		void		(*const *vector_tbl)(const void*);	//fonction of the vector table to run
		const WORD	*delay_lst;							//pointing on the correct delay to apply before next step
		} RAMP;			//define totally a ramp

typedef	struct reel_addr
	{
	BYTE		byte_num;
	BYTE		high_nibble;
	INPUT		opto;				//input linked to the opto sensor
	} REEL_ADDR;

typedef	struct reel_type
	{
	WORD		steps_per_rev;		//total steps in a full rotation
	WORD		steps_per_symbol;	// steps for each symbol */
	BYTE		min_opto;			// minimum opto count which means how often to we check if there is an error with the reel
	BYTE		tolerance;			// how many steps of error per min_opto is acceptable before counting an error
	BYTE		revpos;				// Do we correct the position if the reel spins backward??
	const RAMP	*ramps[MAX_RAMPS];	// ramps
	} REEL_TYPE;


typedef	struct steppervars
	{
	WORD		posn;
	WORD		step_dist;			//	distance in nb of steps from where the reel starts to the final position where the reel will stop.
	WORD		direction;			//	if set to 1 reel is moving forward if -1 reels move backward
	const WORD	*pattern_tbl;		// 	individual pattern  to send to each motor to make it move (pattern of the coils)
	WORD		pattern_idx;		// 	Value to send on reel to energise inductor
	WORD		pixmask;			//	mask: when full step, four values of coil are use (mask=3 ie: 0x00011) when half step, eight values of coil are use (mask=7 ie 0x000111)

	void		(*const *vector_tbl)(const void*);	//individual list of function (the vector table) to be run for individual reel
	const WORD	*delay_tbl;			// 	full list of all the delays defining a ramp
	WORD		delay_idx;			// 	current index into the delay table
	WORD		timer;				//	individual timer to go from one inductor to another

	BYTE		opto_count;			// is set to one when reels pass in front of opt - variable used as a flag to correct position of reel after each full rotation
	WORD		steps_before_optocount;	// Min nb of steps between two count of opto position
	volatile BYTE	spinning;		// state if reel spinning or not
	BYTE		enable;				//Individual flag enabling each reel to be updated during interrupt

	BYTE		located;			//??
	BYTE		pos_corrected;	//number of time the distance to run was modified to correct the position of the reel
	BYTE		tab_missed;		//number of errors each reel committed during all the spinning distance - value increased or not after each full rotation
	BYTE		half_step;
	} STEPPERVARS;

typedef	struct stepperzvars
	{
	WORD		step_posn;		//nb of motors steps from current reel position to opto position updated each step - this number will stay in between 0 and max nb of steps
	WORD		step_from_opto;	//
	BYTE		stop;
	} STEPPERZVARS;


typedef	struct stepper
	{
	const	 REEL_ADDR*		addr;			/*structure showing how to read & write to reel and how where to read opto*/
	const 	 REEL_TYPE*		type;			/*info about the reel no. of steps per rev, nb steps per symbol, RAMP tables ...etc*/
			 STEPPERVARS*	vars;			/*ram associated with the reel*/
			 STEPPERZVARS*	zvars;
	UWORD	tab_posn;						/*position information*/
	} STEPPER;





typedef	struct steppercfg
	{
	const	STEPPER*const*	stepperlist;
	WORD	initsteps;	// No. of steps to wake up steppers
	TIME	initdelay;	// delay between steps
	TIME	optodelay;	//? ?
	WORD	retries;	// not used
	TIME	retrydelay;	// not used
	} STEPCFG;


typedef	struct stepvars
	{
	const	STEPCFG*	config;
	BYTE				enable;			//global flag for all reels to be enable during interrupt
	BYTE				nofmotors;
	} STEPVARS;


typedef	struct stepservices
	{
	WORD		(*getnofmotors)(const	DEVICE* dev);
	void		(*initialisestepper)(const DEVICE* dev);
	void		(*restorepositions)(const DEVICE* dev);
	BYTE		(*motorsidle)(const DEVICE* dev);
	LONG		(*detectoptos)(const DEVICE* dev);

	void		(*spintotab)(const DEVICE* dev,WORD rampidx);
	void		(*spinmotor)(const DEVICE* dev,WORD motidx,WORD steps,WORD rampidx);
	void		(*stopmotor)(const DEVICE* dev,WORD motidx);
	WORD		(*getmotorposn)(const DEVICE* dev,WORD motidx);

	BYTE		(*motoridle)(const DEVICE* dev,WORD motidx);
	BYTE		(*motorspinning)(const DEVICE* dev,WORD motidx);
	WORD		(*gettabsmissed)(const DEVICE* dev,WORD motidx);
	WORD		(*getposncorrected)(const DEVICE* dev,WORD motidx);
	BYTE		(*hardwarefailuredetected)(const DEVICE* dev,WORD motidx);
	void		(*clearerrors)(const DEVICE* dev,WORD motidx);
	void		(*spintozero)(const DEVICE* dev);
	} STEPSERVICES;

/****	Prototypes */

	#include	"stepper.p"

/*************************************************************************/
